@@ -187,6 +187,11 @@ int Device::Write(int fd, const void *buf, int size)
    FD[fd].urb_write_active = 0;
    len = FD[fd].urb_write.actual_length;
 
+#ifdef HPIOD_DEBUG
+   syslog(LOG_INFO, "Write len=%d size=%d\n", len, size);
+   sysdump((void *)buf, len < 32 ? len : 32);
+#endif
+
 bugout:
    return len;
 }
@@ -204,7 +209,10 @@ int Device::Read(int fd, void *buf, int size, int usec)
    }
 
    if (FD[fd].ucnt)
-      return CutBuf(fd, buf, size);
+   {
+      len = CutBuf(fd, buf, size);
+      goto okout;
+   }
 
    gettimeofday (&t1, NULL);     /* get start time */
 
@@ -262,6 +270,12 @@ int Device::Read(int fd, void *buf, int size, int usec)
       break;
    }
 
+okout:
+#ifdef HPIOD_DEBUG
+   syslog(LOG_INFO, "Read len=%d size=%d usec=%d\n", len, size, usec);
+   sysdump(buf, len < 32 ? len : 32);
+#endif
+
 bugout:
    return len;
 }
@@ -287,8 +301,8 @@ int Device::WriteECPChannel(int fd, int value)
    len = usb_control_msg(hd, 
              USB_ENDPOINT_IN | USB_TYPE_VENDOR | USB_RECIP_INTERFACE, /* bmRequestType */
              USB_REQ_GET_STATUS,        /* bRequest */
-             htole16(value),        /* wValue */
-             htole16(interface), /* wIndex */
+             value,        /* wValue */
+             interface, /* wIndex */
              &byte, 1, LIBUSB_CONTROL_REQ_TIMEOUT);
 
    if (len != 1)
@@ -341,28 +355,28 @@ int Device::BridgeChipUp(int fd)
       len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x0758),        /* wValue */
+             0x0758,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
       /* Turn off RLE in auto ECP mode. */
       len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x0a1d),        /* wValue */
+             0x0a1d,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
       /* Place 725 chip in auto ECP mode. */
       len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x0759),        /* wValue */
+             0x0759,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
       /* Force negotiation. */
       len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x0817),        /* wValue */
+             0x0817,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
       /* Read register values. */
@@ -382,7 +396,7 @@ int Device::BridgeChipUp(int fd)
    len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x05ce),        /* wValue */
+             0x05ce,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
    Write(fd, &nullByte, 1);
@@ -391,7 +405,7 @@ int Device::BridgeChipUp(int fd)
    len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x05cd),        /* wValue */
+             0x05cd,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
 
@@ -418,7 +432,7 @@ int Device::BridgeChipDown(int fd)
    len = usb_control_msg(hd, 
              USB_ENDPOINT_OUT | USB_TYPE_VENDOR | USB_RECIP_DEVICE, /* bmRequestType */
              0x04,        /* bRequest */
-             htole16(0x080f),        /* wValue */
+             0x080f,        /* wValue */
              0, /* wIndex */
              NULL, 0, LIBUSB_CONTROL_REQ_TIMEOUT);
    if (len < 0)
@@ -620,7 +634,7 @@ int Device::ClaimInterface(int fd, int config, int interface, int altset)
    }
 
 #ifdef HPIOD_DEBUG
-   syslog(LOG_INFO, "claimed %s interface config=%d interface=%d altset=%d\n", fd == FD_7_1_2 ? "7/1/2" : fd == FD_7_1_3 ? "7/1/3" : "ff/1/1", config, interface, altset);
+   syslog(LOG_INFO, "claimed %s interface config=%d interface=%d altset=%d\n", fd == FD_7_1_2 ? "7/1/2" : fd == FD_7_1_3 ? "7/1/3" : fd == FD_ff_1_1 ? "ff/1/1" : "ff/2/1", config, interface, altset);
 #endif
 
    stat=0;
@@ -646,7 +660,7 @@ int Device::ReleaseInterface(int fd)
    FD[fd].pHD = NULL;
 
 #ifdef HPIOD_DEBUG
-   syslog(LOG_INFO, "released %s interface\n", fd == FD_7_1_2 ? "7/1/2" : fd == FD_7_1_3 ? "7/1/3" : "ff/1/1");
+   syslog(LOG_INFO, "released %s interface\n", fd == FD_7_1_2 ? "7/1/2" : fd == FD_7_1_3 ? "7/1/3" : fd == FD_ff_1_1 ? "ff/1/1" : "ff/2/1");
 #endif
 
    return 0;
@@ -723,7 +737,7 @@ struct usb_device * Device::GetDevice(char *uri)
 int Device::DeviceID(char *buffer, int size)
 {
    usb_dev_handle *hd;
-   int interface;
+   int config,interface,alt;
    int len=0, rlen, maxSize;
 
    if (OpenFD < 0)
@@ -733,7 +747,9 @@ int Device::DeviceID(char *buffer, int size)
    }
 
    hd = FD[OpenFD].pHD;
+   config = FD[OpenFD].Config;
    interface = FD[OpenFD].Interface;
+   alt = FD[OpenFD].AltSetting;
 
    if (hd == NULL)
    {
@@ -748,14 +764,25 @@ int Device::DeviceID(char *buffer, int size)
    rlen = usb_control_msg(hd, 
              USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE, /* bmRequestType */
              USB_REQ_GET_STATUS,        /* bRequest */
-             htole16(0),        /* wValue */
-             htole16(interface), /* wIndex */
+             config,        /* wValue */
+             interface, /* wIndex */    /* note firmware does not follow the USB Printer Class specification for wIndex */
              buffer, maxSize, LIBUSB_CONTROL_REQ_TIMEOUT);
 
    if (rlen < 0)
    {
-      syslog(LOG_ERR, "invalid Device::DeviceID: %m %s %s %d\n", URI,__FILE__, __LINE__);
-      goto bugout;
+      /* Following retry is necessary for a firmware problem with PS A420 products. DES 4/17/07 */
+      syslog(LOG_ERR, "invalid DeviceID wIndex=%x, retrying wIndex=%x: %s %s %d\n", interface, interface << 8, URI,__FILE__, __LINE__);
+      rlen = usb_control_msg(hd, 
+             USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE, /* bmRequestType */
+             USB_REQ_GET_STATUS,        /* bRequest */
+             config,        /* wValue */
+             interface << 8, /* wIndex */ 
+             buffer, maxSize, LIBUSB_CONTROL_REQ_TIMEOUT);
+      if (rlen < 0)
+      {
+         syslog(LOG_ERR, "invalid DeviceID retry ret=%d: %s %s %d\n", rlen, URI,__FILE__, __LINE__);
+         goto bugout;
+      }
    }
 
    len = ntohs(*(short *)buffer);
@@ -795,8 +822,8 @@ int Device::DeviceStatus(unsigned int *status)
    len = usb_control_msg(hd, 
              USB_ENDPOINT_IN | USB_TYPE_CLASS | USB_RECIP_INTERFACE, /* bmRequestType */
              USB_REQ_CLEAR_FEATURE,        /* bRequest */
-             htole16(0),        /* wValue */
-             htole16(interface), /* wIndex */
+             0,        /* wValue */
+             interface, /* wIndex */
              &byte, 1, LIBUSB_CONTROL_REQ_TIMEOUT);
 
    if (len < 0)
@@ -1199,7 +1226,7 @@ Channel *Device::NewChannel(unsigned char sockid, char *sn)
    {
       if (pChannel[i] == NULL)
       {
-         if (sockid == EWS_CHANNEL)
+         if (sockid == EWS_CHANNEL || sockid == SOAPSCAN_CHANNEL)
             pC = new CompChannel(this);
          else if (mode == RAW_MODE)
             pC = new RawChannel(this);  /* constructor sets ClientCnt=1 */
@@ -1279,6 +1306,10 @@ int Device::ChannelOpen(char *sn, int *channel, char *sendBuf, int *result)
    {
       sockid = EWS_CHANNEL;
    }
+   else if (strncasecmp(sn, "hp-soap-scan", 12) == 0)
+   {
+      sockid = SOAPSCAN_CHANNEL;
+   }
    else if (strncasecmp(sn, "echo", 4) == 0)
    {
       sockid = ECHO_CHANNEL;
